"""index_utils Various utilities for working with indexes with mongodb

"""
from navalmartin_mir_db_utils.dbs import MongoDBSession


async def create_collection_index(attribute_name: str,
                                  db_session: MongoDBSession,
                                  collection_name: str,
                                  unique_index: bool = True) -> str:
    """Create an index on the attribute for the given collection

    Parameters
    ----------
    attribute_name: The attribute name to create the index
    db_session: The active database session
    collection_name: The collection name to impose the index on
    unique_index: Whether the index should be unique

    Returns
    -------

    A string with the response of the DB server
    """
    response = await db_session.db[collection_name].create_index(
        keys=attribute_name, unique=unique_index
    )

    return response


import asyncio
import bson
from navalmartin_mir_db_utils.dbs.mongodb_session import MongoDBSession
from navalmartin_mir_db_utils.crud.mongodb_crud_ops import ReadEntityCRUDAPI, CreateEntityCRUDAPI
from navalmartin_mir_db_utils.utils.exceptions import ResourceNotFoundException
from navalmartin_mir_db_utils.crud.crud_utils import get_one_result_or_raise

COLLECTION_NAME = "individuals"
MONGODB_URL = "mongodb://localhost:27021/?replicaSet=mongo-mir-app-set"
MONGO_DB_NAME = "MyDb"

async def run_examples(mir_db_session: MongoDBSession) -> None:

    # insert some data
    response = await CreateEntityCRUDAPI.insert_one(data={"name": "John", "surname": "Doe",
                                                          "city":"SunCity",
                                                          "address": "54 JohnDoe highway"},
                                                    collection_name=COLLECTION_NAME,
                                                    db_session=mir_db_session)

    # get the result back
    read_response = await ReadEntityCRUDAPI.find_one(criteria={'_id': response.inserted_id},
                                                 collection_name=COLLECTION_NAME,
                                                 db_session=mir_db_session,
                                                 projection={})

    print(f"Inserted document {read_response}")
    print("Dropping database")
    await mir_db_session.drop_database()



def main():

    mir_db_session = MongoDBSession(mongodb_url=MONGODB_URL,
                                    db_name=MONGO_DB_NAME,
                                    **{'directConnection': True,
                                       'serverSelectionTimeoutMS': 5})

    print(f"Session to database: {mir_db_session.db_name}")
    print(f"Database: {mir_db_session.mongodb_url}")


    asyncio.run(run_examples(mir_db_session=mir_db_session))

    
if __name__ == '__main__':
    main()



"""module task_performance_result_schema. Models basic schema
for monitoring the performance of a task. The schema is meant
to be embedded in a task document specified by the client code

"""

from pydantic import BaseModel, Field
from typing import List

from navalmartin_mir_db_utils.utils import InvalidTypeEnum


class ErrorType(BaseModel):
    label: str = Field(title="label",
                       description="Identifier for the error",
                       default=InvalidTypeEnum.INVALID.name.upper())
    error_description: str = Field(title="error_description",
                                   description="Textual description of the error",
                                   default=InvalidTypeEnum.INVALID.name.upper())
    details: dict = Field(title="details",
                          description="Details pertaining to the specific error",
                          default={})


class TaskResultSchema(BaseModel):
    errors: List[ErrorType] = Field(title="errors",
                                    description="The errors occurred whilst the task was running",
                                    default=[])
    results: List[dict] = Field(title="results",
                                description="The results obtained by running the task was running",
                                default={})



import time
import psutil
import datetime
import pprint
from navalmartin_mir_db_utils.schemata import TaskPerformanceResultSchema, TaskResultSchema


def sum_task(sleep_time: int, n_elements: int):
    time.sleep(sleep_time)

    total = sum([i for i in range(n_elements)])
    return total


if __name__ == '__main__':
    start_time = time.time()

    pp = pprint.PrettyPrinter(indent=4)
    task_performance = TaskPerformanceResultSchema()
    task_result = TaskResultSchema()

    sum = sum_task(sleep_time=3,
                   n_elements=1000000)

    task_result.results = [{'sum': sum}]

    virtual_mem_dict = dict(psutil.virtual_memory()._asdict())
    cpu_percentage = psutil.cpu_percent()

    pp.pprint(f"Task virtual memory dictionary: {virtual_mem_dict}")
    pp.pprint(f"Task CPU performance: {cpu_percentage}")

    end_time = time.time()
    task_performance.latency = end_time - start_time
    task_performance.ended_at = datetime.datetime.utcnow()
    task_performance.cpu_util = cpu_percentage
    task_performance.disk_util = virtual_mem_dict

    pp.pprint(f"Task performance: {task_performance}")
    pp.pprint(f"Task result: {task_result}")



"""module mongodb_session. Create a session for the MongoDB

"""
import os

import motor.motor_asyncio
from typing import Union

from navalmartin_mir_db_utils.dbs.dbs_utils import DB_ERROR


class MongoDBSession(object):

    @classmethod
    def build_with_arguments(cls, mongodb_url:str, db_name: str, **kwargs) -> "MongoDBSession":
        """Build a MongoDBSession with provided arguments. For a list
        of arguments check https://motor.readthedocs.io/en/stable/api-asyncio/asyncio_motor_client.html

        Parameters
        ----------
        mongodb_url: The MongoDB URL to use
        db_name: The database name to connect to
        kwargs: The arguments to use

        Returns
        -------

        An instance of MongoDBSession
        """

        db_session = MongoDBSession(mongodb_url=mongodb_url,
                                    db_name=db_name,
                                    build_client=False)

        db_session.client = motor.motor_asyncio.AsyncIOMotorClient(mongodb_url, **kwargs)
        db_session.db = db_session.client[db_name]
        return db_session


    def __init__(self, mongodb_url: str = "", db_name: str = "",
                 build_client: bool = True, **kwargs: dict):
        self.mongodb_url: Union[str | None] = mongodb_url
        self.db_name: Union[str | None] = db_name
        self.client = None
        self.db = None

        if build_client:
            if self.mongodb_url == "" or self.mongodb_url is None:
                self.mongodb_url = os.getenv("MONGODB_URL")

            if self.mongodb_url is None or self.mongodb_url == "":
                print(f"{DB_ERROR} The MongoDB URL is None or empty. Check your configuration")
                raise ValueError("MongoDB URL is not set")

            if self.db_name == "" or self.db_name is None:
                self.db_name = os.getenv("MONGODB_NAME")

            if self.db_name is None or self.db_name == "":
                print(f"{DB_ERROR} The DB name is None or empty. Check your configuration")
                raise ValueError("DB name is not set")


            self.client = motor.motor_asyncio.AsyncIOMotorClient(self.mongodb_url, **kwargs)
            self.db = self.client[self.db_name]

    async def drop_database(self) -> None:
        await self.client.drop_database(self.db_name)



def get_db_session() -> MongoDBSession:
    """Get an instance of MongoDBSession. The application
    should load the environment variables 'MONGODB_URL' and
    'MONGODB_NAME'

    Returns
    -------

    """
    return MongoDBSession()


def get_db_session_with_config(config: dict) -> MongoDBSession:
    mongodb_url = config['MONGODB_URL']
    db_name = config['MONGODB_NAME']
    return MongoDBSession(mongodb_url=mongodb_url, db_name=db_name)


